@@ -1,6 +1,5 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import hashlib
 from decimal import Decimal
 from collections import namedtuple, defaultdict
 from itertools import groupby
@@ -20,6 +19,7 @@ from trytond.pool import Pool
 from trytond.rpc import RPC
 from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond.modules.company import CompanyReport
+from trytond.modules.currency.fields import Monetary
 
 from .exceptions import (
     StatementValidateError, StatementValidateWarning, StatementPostError)
@@ -86,24 +86,20 @@ class Statement(Workflow, ModelSQL, ModelView):
         depends=['state', 'company'])
     currency = fields.Function(fields.Many2One(
             'currency.currency', "Currency"), 'on_change_with_currency')
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-        'on_change_with_currency_digits')
     date = fields.Date('Date', required=True, select=True)
-    start_balance = fields.Numeric('Start Balance',
-        digits=(16, Eval('currency_digits', 2)),
-        states=_balance_states, depends=_balance_depends + ['currency_digits'])
-    end_balance = fields.Numeric('End Balance',
-        digits=(16, Eval('currency_digits', 2)),
-        states=_balance_states, depends=_balance_depends + ['currency_digits'])
-    balance = fields.Function(
-        fields.Numeric('Balance',
-            digits=(16, Eval('currency_digits', 2)),
-            states=_balance_states,
-            depends=_balance_depends + ['currency_digits']),
+    start_balance = Monetary(
+        "Start Balance", currency='currency', digits='currency',
+        states=_balance_states, depends=_balance_depends)
+    end_balance = Monetary(
+        "End Balance", currency='currency', digits='currency',
+        states=_balance_states, depends=_balance_depends)
+    balance = fields.Function(Monetary(
+            "Balance", currency='currency', digits='currency',
+            states=_balance_states, depends=_balance_depends),
         'on_change_with_balance')
-    total_amount = fields.Numeric('Total Amount',
-        digits=(16, Eval('currency_digits', 2)),
-        states=_amount_states, depends=_amount_depends + ['currency_digits'])
+    total_amount = Monetary(
+        "Total Amount", currency='currency', digits='currency',
+        states=_amount_states, depends=_amount_depends)
     number_of_lines = fields.Integer('Number of Lines',
         states=_number_states, depends=_number_depends)
     lines = fields.One2Many('account.statement.line', 'statement',
@@ -211,14 +207,6 @@ class Statement(Workflow, ModelSQL, ModelView):
         Date = Pool().get('ir.date')
         return Date.today()
 
-    @staticmethod
-    def default_currency_digits():
-        Company = Pool().get('company.company')
-        if Transaction().context.get('company'):
-            company = Company(Transaction().context['company'])
-            return company.currency.digits
-        return 2
-
     @fields.depends('journal', 'state', 'lines')
     def on_change_journal(self):
         if not self.journal:
@@ -241,12 +229,6 @@ class Statement(Workflow, ModelSQL, ModelView):
         if self.journal:
             return self.journal.currency.id
 
-    @fields.depends('journal')
-    def on_change_with_currency_digits(self, name=None):
-        if self.journal:
-            return self.journal.currency.digits
-        return 2
-
     def get_end_balance(self, name):
         end_balance = self.start_balance
         for line in self.lines:
@@ -504,9 +486,9 @@ class Statement(Workflow, ModelSQL, ModelView):
                 if l.invoice and l.invoice.state in {'cancelled', 'paid'})
 
         if paid_cancelled_invoice_lines:
-            warning_key = ('%s.statement_paid_cancelled_invoice_lines' %
-                    hashlib.md5(str(paid_cancelled_invoice_lines).encode(
-                        'utf-8')).hexdigest())
+            warning_key = Warning.format(
+                'statement_paid_cancelled_invoice_lines',
+                paid_cancelled_invoice_lines)
             if Warning.check(warning_key):
                 raise StatementValidateWarning(warning_key,
                     gettext('account_statement'
@@ -685,14 +667,11 @@ def origin_mixin(_states, _depends):
         number = fields.Char("Number")
         date = fields.Date(
             "Date", required=True, states=_states, depends=_depends)
-        amount = fields.Numeric(
-            "Amount", required=True,
-            digits=(16, Eval('currency_digits', 2)),
-            states=_states, depends=_depends + ['currency_digits'])
+        amount = Monetary(
+            "Amount", currency='currency', digits='currency', required=True,
+            states=_states, depends=_depends)
         currency = fields.Function(fields.Many2One(
                 'currency.currency', "Currency"), 'on_change_with_currency')
-        currency_digits = fields.Function(fields.Integer(
-                "Currency Digits"), 'on_change_with_currency_digits')
         party = fields.Many2One(
             'party.party', "Party", states=_states,
             context={
@@ -743,11 +722,6 @@ def origin_mixin(_states, _depends):
             if self.statement and self.statement.journal:
                 return self.statement.journal.currency.id
 
-        @fields.depends('statement', '_parent_statement.journal')
-        def on_change_with_currency_digits(self, name=None):
-            if self.statement and self.statement.journal:
-                return self.statement.journal.currency.digits
-
     return Mixin
 
 
@@ -786,8 +760,8 @@ class Line(
             'invisible': ~Bool(Eval('origin')),
             },
         domain=[
-            ('statement', '=', Eval('statement')),
-            ('date', '=', Eval('date')),
+            ('statement', '=', Eval('statement', -1)),
+            ('date', '=', Eval('date', None)),
             ],
         depends=['statement', 'date'])
 
@@ -1023,13 +997,10 @@ class LineGroup(ModelSQL, ModelView):
             'Journal'), 'get_journal', searcher='search_journal')
     number = fields.Char('Number')
     date = fields.Date('Date')
-    amount = fields.Numeric('Amount',
-        digits=(16, Eval('currency_digits', 2)),
-        depends=['currency_digits'])
+    amount = Monetary(
+        "Amount", currency='currency', digits='currency')
     currency = fields.Function(fields.Many2One('currency.currency',
             'Currency'), 'get_currency')
-    currency_digits = fields.Function(fields.Integer('Currency Digits'),
-        'get_currency_digits')
     party = fields.Many2One('party.party', 'Party')
     move = fields.Many2One('account.move', 'Move')
 
@@ -1084,9 +1055,6 @@ class LineGroup(ModelSQL, ModelView):
     def get_currency(self, name):
         return self.statement.journal.currency.id
 
-    def get_currency_digits(self, name):
-        return self.statement.journal.currency.digits
-
 
 _states = {
     'readonly': (Eval('statement_state') != 'draft') | Eval('lines', []),
@@ -1106,16 +1074,14 @@ class Origin(origin_mixin(_states, _depends), ModelSQL, ModelView):
                 | ~Eval('statement_state').in_(['draft', 'validated'])),
             },
         domain=[
-            ('statement', '=', Eval('statement')),
-            ('date', '=', Eval('date')),
+            ('statement', '=', Eval('statement', -1)),
+            ('date', '=', Eval('date', None)),
             ],
         depends=['statement', 'date', 'statement_id'])
     statement_id = fields.Function(
         fields.Integer("Statement ID"), 'on_change_with_statement_id')
-    pending_amount = fields.Function(
-        fields.Numeric("Pending Amount",
-            digits=(16, Eval('_parent_statement', {})
-                .get('currency_digits', 2))),
+    pending_amount = fields.Function(Monetary(
+            "Pending Amount", currency='currency', digits='currency'),
         'on_change_with_pending_amount', searcher='search_pending_amount')
     information = fields.Dict(
         'account.statement.origin.information', "Information", readonly=True)
@@ -1200,6 +1166,7 @@ class ImportStatement(Wizard):
         for statement in statements:
             statement.origin_file = fields.Binary.cast(self.start.file_)
         Statement.save(statements)
+        self.start.file_ = None
 
         data = {'res_id': list(map(int, statements))}
         if len(statements) == 1:
